;;                                                                   ;;
;;           Area of polylines, circles and ellipses                 ;;
;;                                                                   ;;
;;             26.oct.2016  -  Gian Paolo Cattaneo                   ;;
;;                                                                   ;;



(defun c:A2 ( / sel :n e1 e1n poly POLY_vl Dx Dy Lp List_vert_poly
                list_p_int P_center dist step1 step2)

    (vla-startundomark (vla-get-activedocument (vlax-get-acad-object)))   
    (if (and
            (princ "\nSelect objects (polylines/circles/ellipses): ")
            (setq sel (ssget '((0 . "LWPOLYLINE,CIRCLE,ELLIPSE"))))
        )
        (progn
            (repeat (setq :n (sslength sel))
                (setq e1 (ssname sel (setq :n (1- :n))))
                (cond
                    ( (= "LWPOLYLINE" (cdr (assoc 0 (entget e1)))) (poly_c e1) )
                    ( t (setq P_center (cdr (assoc 10 (entget e1)))) )
                )
                (entmake
                    (list
                        (cons 0 "TEXT")
                        (cons 8 (getvar "clayer"))
                        (cons 7 (getvar "textstyle"))
                        (cons 10 P_center)
                        (cons 11 P_center)
                        (cons 40 (getvar "textsize"))
                        (cons 72 1)
                        (cons 73 2)
                        (cons 1 (strcat "m" "\\U+00B2 " (rtos (vlax-curve-getArea e1) 2 2)))
                    )
                )
            )
        )
        (alert "No valid object selected")
    )
    (vla-endundomark (vla-get-activedocument (vlax-get-acad-object)))   
    (princ)
)


(defun poly_c ( poly /)

    (setq step1 30) ;--> grid_1 density
    (setq step2 10) ;--> grid_2 density
    (setq jjj 0)    
    (setq POLY_vl (vlax-ename->vla-object POLY))
    (setq list_vert_poly (LM:LWPoly->List POLY))
    (grid_1)   
    (Point_int)
    ;(grid+) ;increase accuracy
    (Point_center)
    (if (= jjj 1) (setq p_prov P_center))
    ;(repeat 2 ;increase accuracy
        (grid_2) 
        (Point_center)
    ;)
)    
   

; Restituisce una griglia di punti all'interno del boundingbox della poly selezionata
(defun grid_1 (/ P1_ P2_ n P> )
    (vla-getboundingbox POLY_vl 'p1 'p2)
    (setq P1_ (vlax-safearray->list p1))
    (setq P2_ (vlax-safearray->list p2))
    (setq P1_ (list (car P1_) (cadr P1_)))
    (setq P2_ (list (car P2_) (cadr P2_)))
    (setq Dx (/ (- (car P2_) (car P1_)) step1))
    (setq Dy (/ (- (cadr P2_) (cadr P1_)) step1))
    (setq n 0)
    (setq P> P1_)
    (setq Lp (list P1_))
    (repeat (* (1+ step1) step1)
        (setq P> (list (+ (car P>) Dx) (cadr P>)))
        (setq Lp (cons P> Lp))
        (setq n (1+ n))
        (if (= n step1)
            (progn
                (setq n 0)
                (setq P1_ (list (car P1_) (+ (cadr P1_) Dy)))
                (setq P> P1_)
                (setq Lp (cons P> Lp))
            )
        )
    )
    (setq Lp (cdr Lp))
    (setq a Lp)
)

   
; Restituisce una griglia di punti intorno al punto centrale (provvisorio)
(defun grid_2 (/ P1_  P> n)
    (setq list_p_int nil)
    (setq P1_ (list (- (car P_center) (* Dx 2)) (- (cadr P_center) (* Dy 2))))
    (setq Dx (/ (* 4 Dx) step2))
    (setq Dy (/ (* 4 Dy) step2))
    (setq n 0)
    (setq P> P1_)
    (setq list_p_int (list P1_))
    (repeat (* (1+ step2) step2)
        (setq P> (list (+ (car P>) Dx) (cadr P>)))
        (setq list_p_int (cons P> list_p_int))
        (setq n (1+ n))
        (if (= n step2)
            (progn
                (setq n 0)
                (setq P1_ (list (car P1_) (+ (cadr P1_) Dy)))
                (setq P> P1_)
                (setq list_p_int (cons P> list_p_int))
            )
        )
    )
    (setq d  list_p_int)
)


; restituisce la lista dei punti interni ad un poligono
; dati:  - lista coordinate dei punti -> Lp
;        - lista coordinate vertici poligono -> list_vert_poly
(defun Point_int (/ P_distant n Pr cont attr p# Pa Pa_ Pb )
    (setq P_distant (list (car (getvar "extmax")) (* 2 (cadr (getvar "extmax")))))    
    (setq list_p_int nil)
    (foreach Pr Lp 
        (setq cont -1)
        (setq attr 0)
        (setq p# nil) 
        (setq Pa (nth (setq cont (1+ cont)) list_vert_poly))
        (setq Pa_ Pa)
        (repeat (length list_vert_poly)
            (setq Pb (nth (setq cont (1+ cont)) list_vert_poly))
            (if (= cont (length list_vert_poly)) (setq Pb Pa_))
            (setq P# (inters Pa Pb Pr P_distant))
            (if (/= P# nil) (setq attr (1+ attr)))
            (setq Pa Pb)
        )
        (if (> (rem attr 2) 0) (setq list_p_int (cons Pr list_p_int)))      
    )
    (setq b list_p_int)
)


; Infittisce la griglia inserendo altri punti
; nel centro delle diagonali tra i punti interni
(defun grid+ (/ G+)
    (setq G+
        (mapcar '(lambda ( x ) (list (+ (car x) (/ Dx 2)) (+ (cadr x) (/ Dy 2)))) list_p_int)
    )
    (setq list_p_int (append G+ list_p_int))
    (setq c list_p_int)
)


; Da una lista di punti restituisce quello pi lontano da un oggetto
; dati:  - lista dei punti -> list_p_int
;        - oggetto -> POLY_vl
(defun Point_center (/ Pa n Pvic)
    (setq Dist 0.0000001)
    (setq P_center nil)
    (foreach Pa list_p_int
    (setq Pvic (vlax-curve-getClosestPointTo POLY_vl Pa))
        (if (> (distance Pa Pvic) Dist)
            (progn
                (setq P_center Pa)
                (setq Dist (distance Pa Pvic))
            )
        )
    )
    (setq jjj (1+ jjj))
    (setq pc P_center)
)


;; LWPolyline to Point List  -  Lee Mac
;; Returns a list of points describing the supplied LWPolyline
(defun LM:LWPoly->List ( ent / der di1 di2 inc lst par rad )
    (setq par 0)
    (repeat (cdr (assoc 90 (entget ent)))
        (if (setq der (vlax-curve-getsecondderiv ent par))
            (if (equal der '(0.0 0.0 0.0) 1e-8)
                (setq lst (cons (vlax-curve-getpointatparam ent par) lst))
                (if
                    (setq rad (distance '(0.0 0.0) (vlax-curve-getfirstderiv ent par))
                          di1 (vlax-curve-getdistatparam ent par)
                          di2 (vlax-curve-getdistatparam ent (1+ par))
                    )
                    (progn
                        (setq inc (/ (- di2 di1) (1+ (fix (* 10 (/ (- di2 di1) rad (+ pi pi)))))))
                        (while (< di1 di2)
                            (setq lst (cons (vlax-curve-getpointatdist ent di1) lst)
                                  di1 (+ di1 inc)
                            )
                        )
                    )
                )
            )
        )
        (setq par (1+ par))
    )
    (setq x lst)
    lst
)


;*************************************************************************

(vl-load-com)

;*************************************************************************

(prompt "\n") (prompt "\n")
(princ "\n\\U+00AB A2.LSP - by Gian Paolo Cattaneo \\U+00BB")
(princ)